@@ -8,6 +8,7 @@
 # author: Zach Chen
 # ver:0.1.1 2021/09/16
 # ver:2.0.1 2022/04/28
+# ver:2.0.2 2022/05/12
 #===========================================
 # output: V6,V7,V8,v9 Raw data
 
@@ -80,13 +81,16 @@ class TrafficMD_I480:
 		print("==============Info=================")
 		print("Output: V6,V7,V8,V9 data:(RAW)")
 		print("V6: Point Cloud Spherical")
-		print("v6 structure: [(range,azimuth,elevation,doppler),......]")
+		print("V6 structure: [(range,azimuth,elevation,doppler,sx,sy,sz),......]")
 		print("V7: Target Object List")
 		print("V7 structure: [(tid,posX,posY,posZ,velX,velY,velZ,accX,accY,accZ),....]")
 		print("V8: Target Index")
 		print("V8 structure: [id1,id2....]")
 		print("V9: Point Cloud Side Info")
 		print("V9 [(snr,noise),....]")
+		print("")
+		print("tlvRead status: {0:'EMPTY',1:'inData',10:'IDLE',99:'FALSE'} ")
+		print("")
 		print("===================================")
 	 
 		
@@ -98,17 +102,82 @@ class TrafficMD_I480:
 		
 	def getHeader(self):
 		return self.hdr
-		
+	
+	#################################################################
+	# for version manegement
+	def jb_u32ToBytes(self, v): # convert u32 as Bytes
+		A = np.zeros( (4, 1) )
+		for i in range(4):
+			A[3 - i] = int(v  % 256)
+			v = int(v / 256)
+		return int(A[0, 0]), int(A[1, 0]), int(A[2, 0]), int(A[3, 0])
+	
+	def jb_versionManagement(self):
+		# test, for example: 
+		# result as following,
+		# (1) Platform    : IWR ISK, JB, 1843
+		# (2) FW   version: V80.61R
+		# (3) TOOL version: I.04.08.00
+		#
+		# (1) Platform
+		vHardware, vMagic, vType, vRxTxNum = self.jb_u32ToBytes(self.hdr.platform)			
+		# hardware type
+		if int(vHardware / 16) == 0:
+			tmp = ''
+		elif int(vHardware / 16) == 1:
+			tmp = 'IWR '
+		elif int(vHardware / 16) == 2:
+			tmp = 'AWR '
+		else:
+			tmp = ''
+		# hardware antenna
+		if int(vHardware % 16) == 0:
+			tmp += ''
+		elif int(vHardware % 16) == 1:
+			tmp += 'ISK'
+		elif int(vHardware % 16) == 2:
+			tmp += 'AOP'
+		else:
+			tmp += ''
+		vHardware = tmp	
+		# sel magic
+		if int(vMagic % 16) == 15:
+			vMagic = 'JB'
+		else:
+			vMagic = ''			
+		print("Platform    : {}, {}, {:02x}{:02x}".format(vHardware, vMagic, vType, vRxTxNum))
+		# (2) FW version
+		vAnt, vMagic, vMajor, vMinor = self.jb_u32ToBytes(self.hdr.chirpMargin)			
+		# hardware antenna
+		if vAnt == 'I':
+			vAnt = ', Ant= ISK'
+		elif vAnt == 'A':
+			vAnt = ', Ant= AOP'
+		else:	
+			vAnt = ''
+		print("FW   version: V{:02d}.{:02d}{:c} {}".format(vMajor, vMinor, vMagic, vAnt))
+		# (3) TOOL version
+		vTool, vMajor, vMinor, vMicro = self.jb_u32ToBytes(self.hdr.frameMargin)			
+		print("TOOL version: {:c}.{:02d}.{:02d}.{:02d}".format(vTool, vMajor, vMinor, vMicro))
+	#################################################################
+				
 	def headerShow(self):
 		print("***Header***********") 
 		print("Version:     \t%x "%(self.hdr.version))
-		print("Platform:    \t%X "%(self.hdr.platform))
+		#print("Platform:    \t%X "%(self.hdr.platform))
 		print("TimeStamp:   \t%s "%(self.hdr.timeStamp))
 		print("TotalPackLen:\t%d "%(self.hdr.totalPackLen))
 		print("PID(frame#): \t%d "%(self.hdr.frameNumber))
 		print("subframe#  : \t%d "%(self.hdr.subFrameNumber))
-		print("Inter-chirp Processing Margin:\t{:d} us".format(self.hdr.chirpMargin))
-		print("Inter-frame Processing Margin:\t{:d} us".format(self.hdr.frameMargin))
+	    ########################################################
+		# original
+		#print("Platform:    \t%X "%(self.hdr.platform))		
+		#print("Inter-chirp Processing Margin:\t{:d} us".format(self.hdr.chirpMargin))
+		#print("Inter-frame Processing Margin:\t{:d} us".format(self.hdr.frameMargin))
+		# new replaced
+		self.jb_versionManagement()
+	    ########################################################
+		
 		print("Inter-frame Processing Time:\t{:d} us".format(self.hdr.trackProcessTime))
 		print("UART Send Time:\t{:d} us".format(self.hdr.uartSendTime))
 		print("numTLVs:     \t%d "%(self.hdr.numTLVs))
@@ -195,6 +264,12 @@ class TrafficMD_I480:
 #  v7: Target Object List information
 #  v8: Target Index information
 #  v9: DPIF Point Cloud Side Infomation
+#  
+#   0: empty: tlv = 0
+#   1: data output
+#  
+#   10: idle
+#   99: error
 #
 	def tlvRead(self,disp,df = None):
 		#print("---tlvRead---")
@@ -223,7 +298,7 @@ class TrafficMD_I480:
 				ch = self.port.read()
 			except:
 				#return (False,v6,v7,v8,v9)
-				return self.list2df(False,v6,v7,v8,v9) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
+				return self.list2df(99,v6,v7,v8,v9) if (df == 'DataFrame') else (99,v6,v7,v8,v9)
 			#print(str(ch))
 			if lstate == 'idle':
 				#print(self.magicWord)
@@ -240,7 +315,7 @@ class TrafficMD_I480:
 					idx = 0
 					rangeProfile = b""
 					#return (False,v6,v7,v8,v9)
-					return self.list2df(False,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
+					return self.list2df(10,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (10,v6,v7,v8,v9)
 		
 			elif lstate == 'header':
 				sbuf += ch
@@ -260,7 +335,7 @@ class TrafficMD_I480:
 						if self.dbg == True:
 							print("(Header)Improper TLV structure found: ")
 						#return (False,v6,v7,v8,v9)
-						return self.list2df(False,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
+						return self.list2df(99,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (99,v6,v7,v8,v9)
 					
 					if disp == True:  
 						self.headerShow()
@@ -268,7 +343,8 @@ class TrafficMD_I480:
 					tlvCount = self.hdr.numTLVs
 					if self.hdr.numTLVs == 0:
 						#return (False,v6,v7,v8,v9)
-						return self.list2df(False,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
+						#return self.list2df(False,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
+						return self.list2df(0,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (0,v6,v7,v8,v9)
 						
 					if self.sm == True:
 						print("(Header)")
@@ -281,7 +357,7 @@ class TrafficMD_I480:
 					idx = 0
 					lstate = 'idle'
 					#return (False,v6,v7,v8,v9)
-					return self.list2df(False,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
+					return self.list2df(99,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (99,v6,v7,v8,v9)
 					
 			elif lstate == 'TL': #TLV Header type/length
 				sbuf += ch
@@ -300,7 +376,7 @@ class TrafficMD_I480:
 							lstate = 'idle'
 							self.port.flushInput()
 							#return (False,v6,v7,v8,v9)
-							return self.list2df(False,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
+							return self.list2df(99,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (99,v6,v7,v8,v9)
 							
 					except:
 						print("TL unpack Improper Data Found:")
@@ -309,7 +385,7 @@ class TrafficMD_I480:
 						self.port.flushInput()
 						#return (False,v6,v7,v8,v9)
 					
-						return self.list2df(False,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
+						return self.list2df(99,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (99,v6,v7,v8,v9)
 					
 					unitByteCount,lstate ,plen ,dataBytes,lenCount, numOfPoints = self.tlvTypeInfo(ttype,self.tlvLength,disp)
 					#print("dataBytes={:d} lenCount={:d}".format(dataBytes , lenCount))
@@ -328,21 +404,22 @@ class TrafficMD_I480:
 						#print(":".join("{:02x}".format(c) for c in sbuf))
 						(r,a,e,d) = struct.unpack('4f', sbuf)
 						#print("({:2d}:{:4d})(idx:({:4d}) elv:{:.4f} azimuth:{:.4f} doppler:{:.4f} range:{:.4f}".format(numOfPoints,lenCount,idx,e,a,d,r))
+						sz  = r * np.sin(e)
+						sx  = r * np.cos(e) * np.sin(a)
+						sy  = r * np.cos(e) * np.cos(a)
 						if (df == 'DataFrame'):
-							sz  = r * np.sin(e)
-							sx  = r * np.cos(e) * np.sin(a)
-							sy  = r * np.cos(e) * np.cos(a)
+							
 							#fieldNames = ['time','frameNum','type','range/px','azimuth/py','elv/vx','doppler/vy','sx/accX','sy/accY','sz/pz','na/vz','na/accZ','na/ID']
 							v6df.append((self.hdr.frameNumber,'v6',r,a,e,d,sx,sy,sz))
 						else:
-							v6.append((r,a,e,d))
+							v6.append((r,a,e,d,sx,sy,sz))
 							
 						sbuf = b""
 					except:
 						if self.dbg == True:
 							print("(6.1)Improper Type 6 Value structure found: ")
 						#return (False,v6,v7,v8,v9)
-						return self.list2df(False,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
+						return self.list2df(99,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (99,v6,v7,v8,v9)
 					
 				if idx == lenCount:
 					if disp == True:
@@ -355,7 +432,7 @@ class TrafficMD_I480:
 							print("(V6:tlvCnt={:d})=>(idle) :true".format(tlvCount))
 						#return (True,v6,v7,v8,v9)
 						print("(lib)v6=============SaveAppend==============")
-						return self.list2df(True,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (True,v6,v7,v8,v9)
+						return self.list2df(1,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (1,v6,v7,v8,v9)
 						
 					else: # Go to TL to get others type value
 						lstate = 'TL' #'tlTL'
@@ -367,7 +444,7 @@ class TrafficMD_I480:
 					sbuf = b""
 					lstate = 'idle'
 					#return (False,v6,v7,v8,v9)
-					return self.list2df(False,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
+					return self.list2df(99,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (99,v6,v7,v8,v9)
 					
 			elif lstate == 'V9':  
 				sbuf += ch
@@ -388,7 +465,7 @@ class TrafficMD_I480:
 						if self.dbg == True:
 							print("(7)Improper Type 9 Value structure found: ")
 						#return (False,v6,v7,v8,v9)
-						return self.list2df(False,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
+						return self.list2df(99,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (99,v6,v7,v8,v9)
 						
 				if idx >= lenCount:
 					if disp == True:
@@ -400,7 +477,7 @@ class TrafficMD_I480:
 						if self.sm == True:
 							print("(V9)=>(idle) :true")
 						#return (True,v6,v7,v8,v9)
-						return self.list2df(True,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (True,v6,v7,v8,v9)
+						return self.list2df(1,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (1,v6,v7,v8,v9)
 						
 					else: # Go to TL to get others type value
 						lstate = 'TL'
@@ -413,28 +490,28 @@ class TrafficMD_I480:
 					lstate = 'idle'
 					sbuf = b""
 					#return (False,v6,v7,v8,v9)
-					return self.list2df(False,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
+					return self.list2df(99,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (99,v6,v7,v8,v9)
 					
 			elif lstate == 'V7':
 				sbuf += ch
 				idx += 1
 				if (idx % dataBytes == 0):
 					
-					print("V7:dataBytes({:d}) lenCount({:d}) index:{:d}".format(dataBytes,lenCount,idx))
+					#print("V7:dataBytes({:d}) lenCount({:d}) index:{:d}".format(dataBytes,lenCount,idx))
 					try:
 						(tid,posX,posY,posZ,velX,velY,velZ,accX,accY,accZ,ec0,ec1,ec2,ec3,ec4,ec5,ec6,ec7,ec8,ec9,ec10,ec11,ec12,ec13,ec14,ec15,g,conf) = struct.unpack('I27f', sbuf) 
 						#fieldNames = ['time','frameNum','type','range/px','azimuth/py','elv/vx','doppler/vy','sx/accX','sy/accY','sz/pz','na/vz','na/accZ','na/ID']
 						if (df == 'DataFrame' ):
 							v7df.append((self.hdr.frameNumber,'v7',posX,posY,posZ,velX,velY,velZ,accX,accY,accZ,tid))
 						else:
-							v7.append((tid,posX,posY,posZ,velX,velY,velZ,accX,accY,accZ))
+							v7.append((posX,posY,posZ,velX,velY,velZ,accX,accY,accZ,tid))
 							#print("tid = ({:d}) ,posX:{:.4f} posY:{:.4f} posZ:{:.4f}".format(tid,posX,posY,posZ))
 						sbuf = b""
 					except:
 						if self.dbg == True:
 							print("(7)Improper Type 7 Value structure found: ")
 						#return (False,v6,v7,v8,v9)
-						return self.list2df(False,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
+						return self.list2df(99,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (99,v6,v7,v8,v9)
 						
 				if idx >= lenCount:
 					if disp == True:
@@ -446,7 +523,7 @@ class TrafficMD_I480:
 						if self.sm == True:
 							print("(V7)=>(idle) :true")
 						#return (True,v6,v7,v8,v9)
-						return self.list2df(True,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (True,v6,v7,v8,v9)
+						return self.list2df(1,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (1,v6,v7,v8,v9)
 						
 					else: # Go to TL to get others type value
 						lstate = 'TL'
@@ -459,7 +536,7 @@ class TrafficMD_I480:
 					lstate = 'idle'
 					sbuf = b""
 					#return (False,v6,v7,v8,v9)
-					return self.list2df(False,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
+					return self.list2df(99,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (99,v6,v7,v8,v9)
 				
 			elif lstate == 'V8':
 				idx += 1
@@ -477,21 +554,21 @@ class TrafficMD_I480:
 						v8o = [self.hdr.frameNumber,'v8']
 						v8df = v8o.extend(v8)
 					#return (True,v6,v7,v8,v9)
-					return self.list2df(True,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (True,v6,v7,v8,v9)
+					return self.list2df(1,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (1,v6,v7,v8,v9)
 				
 				if idx > lenCount:
 					sbuf = b""
 					idx = 0
 					lstate = 'idle'
 					#return (False,v6,v7,v8,v9)
-					return self.list2df(False,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (False,v6,v7,v8,v9)
+					return self.list2df(99,v6df,v7df,v8df,v9df) if (df == 'DataFrame') else (99,v6,v7,v8,v9)
 
 	def getRecordData(self,frameNum):
 		s_fn = frameNum + self.sim_startFN
 		#print("frame number:{:}".format(s_fn))
 		v6d = self.v6simo[self.v6simo['fN'] == s_fn]
 		#v6d =  v6dd[v6dd['doppler'] < 0.0]
-		#print(v6d)
+		#print("==============v6d lib==================")
 		v7d = self.v7simo[self.v7simo['fN'] == s_fn]
 		v8d = self.v8simo[self.v8simo['fN'] == s_fn]
 		v9d = self.v9simo[self.v9simo['fN'] == s_fn]
@@ -541,7 +618,7 @@ class TrafficMD_I480:
 		df7 = pd.read_csv(self.fileName, names = self.v7_col_names, skiprows = [0])  
 		
 		v7simc = df7[df7['type'] == 'v7']
-		self.v7simo  = v7simc.loc[:,['fN','posX','posY','velX','velY','accX','accY','posZ','velZ','accZ','tid']]
+		self.v7simo  = v7simc.loc[:,['fN','type','posX','posY','velX','velY','accX','accY','posZ','velZ','accZ','tid']]
 		
 		v8simc = df[df['type'] == 'v8']
 		self.v8simo  = v8simc.loc[:,['fN','elv']]
